/* --COPYRIGHT--,BSD
 * Copyright (c) 2014, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
/*  
 * ======== main.c ========
 * LED Control Demo:
 *
 * This example implements a simple command-line interface, where the command 
 * is ended by pressing return.  It accepts four "commands":
 * "LED ON"
 * "LED OFF"
 * "LED TOGGLE - SLOW"
 * "LED TOGGLE  FAST"
 +----------------------------------------------------------------------------+
 * Please refer to the Examples Guide for more details.
 * ---------------------------------------------------------------------------*/

// User Libraries
#include "console.h"
#include "driverlib.h"
#include "hal.h"
#include "USB_config/descriptors.h"
#include "USB_API/USB_Common/device.h"
#include "USB_API/USB_Common/usb.h"                 // USB-specific functions
#include "USB_API/USB_CDC_API/UsbCdc.h"
#include "USB_app/usbConstructs.h"

// C Standard Libraries
#include <string.h>
/*
 * NOTE: Modify hal.h to select a specific evaluation board and customize for
 * your own board.
 */


//Internal Function declarations
void USBtask(void);
void ADCtask(void);

// Global flags set by events
volatile unsigned char g_bCDCDataReceived_event = FALSE;	// Indicates data has been rx'ed
															// without an open rx operation
// Global Variables
char wholeString[MAX_CONSOLE_STR_LEN] = ""; 				// Entire input str from last 'return'


/*  
 * ======== main ========
 */
void main (void)
{
	WDT_A_hold(WDT_A_BASE); 			// Stop watchdog timer
    PMM_setVCore(PMM_CORE_LEVEL_2);		// Minumum Vcore setting required for the USB API is PMM_CORE_LEVEL_2
    initPorts();           				// Config GPIOs for low-power (output low)
    initClocks();						// Config system clocks
//	initTimer(); 						// Prepare timer for LED toggling
    initSPI();							// Configure
    USB_setup(TRUE, TRUE); 				// Init USB & events; if a host is present, connect
    __enable_interrupt();  				// Enable interrupts globally
    
    while(1)
    {
        USBtask();
        //ADCtask();
    }

} //end: main()



void USBtask(void)
{
	char pieceOfString[MAX_CONSOLE_STR_LEN] = "";	// Holds the new addition to the string

	// Check the USB state and directly main loop accordingly
	switch (USB_connectionState())
	{
		// This case is executed while your device is enumerated on the
		// USB host
		case ST_ENUM_ACTIVE:

			// Enter LPM0 (can't do LPM3 when active)
			//__bis_SR_register(LPM0_bits + GIE);
			//_NOP();
			// Exit LPM on USB receive and perform a receive operation

			// If true, some data is in the buffer; begin receiving a cmd
			if (g_bCDCDataReceived_event)
			{
				g_bCDCDataReceived_event = FALSE;

				// Holds the outgoing string
				//char outString[MAX_STR_LENGTH] = ""; //Defined in console.h

				// Add bytes in USB buffer to the string
				cdcReceiveDataInBuffer((uint8_t*)pieceOfString,
					MAX_CONSOLE_STR_LEN,
					CDC0_INTFNUM); // Get the next piece of the string

				// Append new piece to the whole
				strcat(wholeString,pieceOfString);
				//Ignore old characters
				//strcpy(wholeString,pieceOfString);

				// Echo back the characters received
				cdcSendDataInBackground((uint8_t*)pieceOfString,
					strlen(pieceOfString),CDC0_INTFNUM,0);

				// Has the user pressed return yet?
				if (retInString(wholeString))
				{
					console(wholeString);			// Process command & print response
					printCommandPrompt();			// Print command prompt
					for (int i = 0; i < MAX_CONSOLE_STR_LEN; i++)
						wholeString[i] = 0x00;		// Clear the string in preparation for the next one
				}
			}
			break;

		// These cases are executed while your device is disconnected from
		// the host (meaning, not enumerated); enumerated but suspended
		// by the host, or connected to a powered hub without a USB host
		// present.
		case ST_PHYS_DISCONNECTED:
		case ST_ENUM_SUSPENDED:
		case ST_PHYS_CONNECTED_NOENUM_SUSP:

			//Turn off LED P1.0
			GPIO_setOutputLowOnPin(LED_PORT, LED_PIN);
			//__bis_SR_register(LPM3_bits + GIE);
			_NOP();
			break;

		// The default is executed for the momentary state
		// ST_ENUM_IN_PROGRESS.  Usually, this state only last a few
		// seconds.  Be sure not to enter LPM3 in this state; USB
		// communication is taking place here, and therefore the mode must
		// be LPM0 or active-CPU.
		case ST_ENUM_IN_PROGRESS:
		default:;
	}
}


#if 0
void ADCtask(void)
{
	if(g_DRDY_flag)
	{
		g_DRDY_flag = 0;
		GPIO_disableInterrupt(GPIO_PORT_P1, GPIO_PIN2);

		ADS126xReadDataCpp(&writeBuffer[writeBufferIdx],g_excitationStatus);

		if((g_excitationStatus) && (g_excitationCounter >= g_samplesPerExcitation))
		{
			if(g_excitationStatus==EXC_POS)
			{
				set_excitation(EXC_NEG);
				g_excitationCounter=0;
			}
			else if(g_excitationStatus==EXC_NEG)
			{
				set_excitation(EXC_POS);
				g_excitationCounter=0;
			}
		}

		if(writeBuffer[writeBufferIdx].data.status & 0x80)
		{
			writeBufferIdx++;
			ADS126xReadADC2Data(&writeBuffer[writeBufferIdx]);
		}

		writeBufferIdx++;
		g_excitationCounter++;

		GPIO_enableInterrupt(GPIO_PORT_P1, GPIO_PIN2);
	}
}
#endif

